@@ -9,6 +9,7 @@ import random
 import re
 import time
 from xml.etree import ElementTree
+from pkg_resources import DistributionNotFound, get_distribution, parse_version
 
 import requests
 
@@ -30,6 +31,13 @@ except ImportError:
     # Python 2.X
     from urllib import urlencode
 
+try:
+    # Python 3.X
+    from urllib.parse import quote
+except ImportError:
+    # Python 2.X
+    from urllib import quote
+
 try:
     # Python 3.X
     import html.entities as htmlentities
@@ -55,11 +63,17 @@ except NameError:
 
 __author__ = 'Daniel Lindsley, Joseph Kocherhans, Jacob Kaplan-Moss'
 __all__ = ['Solr']
-__version__ = (3, 5, 0)
 
+try:
+    pkg_distribution = get_distribution(__name__)
+    __version__ = pkg_distribution.version
+    version_info = pkg_distribution.parsed_version
+except DistributionNotFound:
+    __version__ = '0.0.dev0'
+    version_info = parse_version(__version__)
 
 def get_version():
-    return "%s.%s.%s" % __version__[:3]
+    return __version__
 
 
 DATETIME_REGEX = re.compile('^(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})T(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})(\.\d+)?Z$')
@@ -304,7 +318,9 @@ class Solr(object):
         solr = pysolr.Solr('http://localhost:8983/solr', results_cls=dict)
 
     """
-    def __init__(self, url, decoder=None, timeout=60, results_cls=Results, search_handler='select', use_qt_param=False):
+
+    def __init__(self, url, decoder=None, timeout=60, results_cls=Results, search_handler='select', use_qt_param=False, always_commit=False,
+                 auth=None, verify=True):
         self.decoder = decoder or json.JSONDecoder()
         self.url = url
         self.timeout = timeout
@@ -313,11 +329,15 @@ class Solr(object):
         self.results_cls = results_cls
         self.search_handler = search_handler
         self.use_qt_param = use_qt_param
+        self.auth = auth
+        self.verify = verify
+        self.always_commit = always_commit
 
     def get_session(self):
         if self.session is None:
             self.session = requests.Session()
             self.session.stream = False
+            self.session.verify = self.verify
         return self.session
 
     def _get_log(self):
@@ -360,10 +380,9 @@ class Solr(object):
 
         if bytes_body is not None:
             bytes_body = force_bytes(body)
-
         try:
             resp = requests_method(url, data=bytes_body, headers=headers, files=files,
-                                   timeout=self.timeout)
+                                   timeout=self.timeout, auth=self.auth)
         except requests.exceptions.Timeout as err:
             error_message = "Connection to server '%s' timed out: %s"
             self.log.error(error_message, url, err, exc_info=True)
@@ -430,7 +449,7 @@ class Solr(object):
     def _suggest_terms(self, params, handler='terms'):
         return self._select(params, handler)
 
-    def _update(self, message, clean_ctrl_chars=True, commit=True, softCommit=False, waitFlush=None, waitSearcher=None,
+    def _update(self, message, clean_ctrl_chars=True, commit=None, softCommit=False, waitFlush=None, waitSearcher=None,
                 overwrite=None, handler='update'):
         """
         Posts the given xml message to http://<self.url>/update and
@@ -454,6 +473,9 @@ class Solr(object):
 
         path = '%s/' % path_handler
 
+        if commit is None:
+            commit = self.always_commit
+
         if commit:
             query_vars.append('commit=%s' % str(bool(commit)).lower())
         elif softCommit:
@@ -808,7 +830,7 @@ class Solr(object):
                 continue
 
             # To avoid multiple code-paths we'd like to treat all of our values as iterables:
-            if isinstance(value, (list, tuple)):
+            if isinstance(value, (list, tuple, set)):
                 values = value
             else:
                 values = (value, )
@@ -817,6 +839,11 @@ class Solr(object):
                 if self._is_null_value(bit):
                     continue
 
+                if key == '_doc':
+                    child = self._build_doc(bit, boost)
+                    doc_elem.append(child)
+                    continue
+
                 attrs = {'name': key}
 
                 if fieldUpdates and key in fieldUpdates:
@@ -832,7 +859,7 @@ class Solr(object):
 
         return doc_elem
 
-    def add(self, docs, boost=None, fieldUpdates=None, commit=True, softCommit=False, commitWithin=None, waitFlush=None,
+    def add(self, docs, boost=None, fieldUpdates=None, commit=None, softCommit=False, commitWithin=None, waitFlush=None,
             waitSearcher=None, overwrite=None, handler='update'):
         """
         Adds or updates documents.
@@ -840,7 +867,7 @@ class Solr(object):
         Requires ``docs``, which is a list of dictionaries. Each key is the
         field name and each value is the value to index.
 
-        Optionally accepts ``commit``. Default is ``True``.
+        Optionally accepts ``commit``. Default is ``None``. None signals to use default
 
         Optionally accepts ``softCommit``. Default is ``False``.
 
@@ -890,12 +917,13 @@ class Solr(object):
         return self._update(m, commit=commit, softCommit=softCommit, waitFlush=waitFlush, waitSearcher=waitSearcher,
                             overwrite=overwrite, handler=handler)
 
-    def delete(self, id=None, q=None, commit=True, softCommit=False, waitFlush=None, waitSearcher=None, handler='update'):
+    def delete(self, id=None, q=None, commit=None, softCommit=False, waitFlush=None, waitSearcher=None, handler='update'):
         """
         Deletes documents.
 
         Requires *either* ``id`` or ``query``. ``id`` is if you know the
-        specific document id to remove. ``query`` is a Lucene-style query
+        specific document id to remove. Note that ``id`` can also be a list of
+        document ids to be deleted. ``query`` is a Lucene-style query
         indicating a collection of documents to delete.
 
         Optionally accepts ``commit``. Default is ``True``.
@@ -909,6 +937,7 @@ class Solr(object):
         Usage::
 
             solr.delete(id='doc_12')
+            solr.delete(id=['doc_1', 'doc_3'])
             solr.delete(q='*:*')
 
         """
@@ -917,7 +946,14 @@ class Solr(object):
         elif id is not None and q is not None:
             raise ValueError('You many only specify "id" OR "q", not both.')
         elif id is not None:
-            m = '<delete><id>%s</id></delete>' % id
+            if not isinstance(id, (list, set, tuple)):
+                id = [id]
+            else:
+                id = list(filter(None, id))
+            if id:
+                m = '<delete>%s</delete>' % ''.join('<id>%s</id>' % i for i in id)
+            else:
+                raise ValueError('The list of documents to delete was empty.')
         elif q is not None:
             m = '<delete><query>%s</query></delete>' % q
 
@@ -1003,13 +1039,13 @@ class Solr(object):
             "wt": "json",
         }
         params.update(kwargs)
-
+        filename = quote(file_obj.name.encode('utf-8'))
         try:
             # We'll provide the file using its true name as Tika may use that
             # as a file type hint:
             resp = self._send_request('post', handler,
                                       body=params,
-                                      files={'file': (file_obj.name, file_obj)})
+                                      files={'file': (filename, file_obj)})
         except (IOError, SolrError) as err:
             self.log.error("Failed to extract document metadata: %s", err,
                            exc_info=True)
@@ -1022,10 +1058,10 @@ class Solr(object):
                            exc_info=True)
             raise
 
-        data['contents'] = data.pop(file_obj.name, None)
+        data['contents'] = data.pop(filename, None)
         data['metadata'] = metadata = {}
 
-        raw_metadata = data.pop("%s_metadata" % file_obj.name, None)
+        raw_metadata = data.pop("%s_metadata" % filename, None)
 
         if raw_metadata:
             # The raw format is somewhat annoying: it's a flat list of
@@ -1174,10 +1210,14 @@ def sanitize(data):
 
 class SolrCloud(Solr):
 
-    def __init__(self, zookeeper, collection, decoder=None, timeout=60, retry_timeout=0.2, *args, **kwargs):
+    def __init__(self, zookeeper, collection, decoder=None, timeout=60, retry_timeout=0.2, auth=None, verify=True,
+                 *args, **kwargs):
         url = zookeeper.getRandomURL(collection)
+        self.auth = auth
+        self.verify = verify
 
-        super(SolrCloud, self).__init__(url, decoder=decoder, timeout=timeout, *args, **kwargs)
+        super(SolrCloud, self).__init__(url, decoder=decoder, timeout=timeout, auth=self.auth, verify = self.verify,
+                                        *args, **kwargs)
 
         self.zookeeper = zookeeper
         self.collection = collection
@@ -1280,7 +1320,7 @@ class ZooKeeper(object):
 
         hosts = []
         if collname not in self.collections:
-            raise SolrError("Unknown collection: %s", collname)
+            raise SolrError("Unknown collection: %s" % collname)
         collection = self.collections[collname]
         shards = collection[ZooKeeper.SHARDS]
         for shardname in shards.keys():
