@@ -2,9 +2,11 @@
 from __future__ import absolute_import, unicode_literals
 
 import datetime
+import random
 import unittest
 from io import StringIO
 from xml.etree import ElementTree
+import random
 
 from pysolr import (NESTED_DOC_KEY, Results, Solr, SolrError, clean_xml_string,
                     force_bytes, force_unicode, json, safe_urlencode, sanitize,
@@ -20,6 +22,11 @@ try:
 except ImportError:
     from urllib import unquote_plus
 
+try:
+    from urllib.parse import quote
+except ImportError:
+    from urllib import quote
+
 
 class UtilsTestCase(unittest.TestCase):
     def test_unescape_html(self):
@@ -131,7 +138,12 @@ class ResultsTestCase(unittest.TestCase):
         self.assertEqual(to_iter[2], {'id': 3})
 
 
-class SolrTestCase(unittest.TestCase):
+class SolrTestCaseMixin(object):
+    def get_solr(self, collection, timeout=60, always_commit=False):
+        return Solr('http://localhost:8983/solr/%s' % collection, timeout=timeout, always_commit=always_commit)
+
+
+class SolrTestCase(unittest.TestCase, SolrTestCaseMixin):
     def setUp(self):
         super(SolrTestCase, self).setUp()
         self.solr = self.get_solr("core0")
@@ -213,12 +225,12 @@ class SolrTestCase(unittest.TestCase):
         ]
 
         # Clear it.
-        self.solr.delete(q='*:*')
+        self.solr.delete(q='*:*', commit=True)
 
         # Index our docs. Yes, this leans on functionality we're going to test
         # later & if it's broken, everything will catastrophically fail.
         # Such is life.
-        self.solr.add(self.docs)
+        self.solr.add(self.docs, commit=True)
 
         # Mock the _send_request method on the solr instance so that we can
         # test that custom handlers are called correctly.
@@ -230,16 +242,17 @@ class SolrTestCase(unittest.TestCase):
         # slash handling are caught quickly:
         return self.assertEqual(URL, '%s/%s' % (self.solr.url.replace('/core0', ''), path))
 
-    def get_solr(self, collection, timeout=60):
-        return Solr('http://localhost:8983/solr/%s' % collection, timeout=timeout)
+    def get_solr(self, collection, timeout=60, always_commit=False):
+        return Solr('http://localhost:8983/solr/%s' % collection, timeout=timeout, always_commit=always_commit)
 
     def test_init(self):
         self.assertEqual(self.solr.url, 'http://localhost:8983/solr/core0')
         self.assertTrue(isinstance(self.solr.decoder, json.JSONDecoder))
         self.assertEqual(self.solr.timeout, 60)
 
-        custom_solr = self.get_solr("core0", timeout=17)
+        custom_solr = self.get_solr("core0", timeout=17, always_commit=True)
         self.assertEqual(custom_solr.timeout, 17)
+        self.assertEqual(custom_solr.always_commit, True)
 
     def test_custom_results_class(self):
         solr = Solr('http://localhost:8983/solr/core0', results_cls=dict)
@@ -544,6 +557,50 @@ class SolrTestCase(unittest.TestCase):
         self.assertTrue('<field name="id">doc_1</field>' in doc_xml)
         self.assertEqual(len(doc_xml), 152)
 
+    def test__build_doc_with_sets(self):
+        doc = {
+            'id': 'doc_1',
+            'title': 'Set test doc',
+            'tags': set(['alpha', 'beta']),
+        }
+        doc_xml = force_unicode(ElementTree.tostring(self.solr._build_doc(doc), encoding='utf-8'))
+        self.assertTrue('<field name="id">doc_1</field>' in doc_xml)
+        self.assertTrue('<field name="title">Set test doc</field>' in doc_xml)
+        self.assertTrue('<field name="tags">alpha</field>' in doc_xml)
+        self.assertTrue('<field name="tags">beta</field>' in doc_xml)
+        self.assertEqual(len(doc_xml), 144)
+
+    def test__build_doc_with_sub_docs(self):
+        sub_docs = [
+            {
+                'id': 'sub_doc_1',
+                'title': 'Example sub doc ☃ 1',
+                'price': 1.59,
+                'popularity': 4
+            },
+            {
+                'id': 'sub_doc_2',
+                'title': 'Example sub doc ☃ 2',
+                'price': 21.13,
+                'popularity': 1
+            },
+        ]
+        doc = {
+            'id': 'doc_1',
+            'title': 'Example doc ☃ 1',
+            'price': 12.59,
+            'popularity': 10,
+            '_doc': sub_docs
+        }
+        doc_xml = self.solr._build_doc(doc)
+        self.assertEqual(doc_xml.find("*[@name='id']").text, doc['id'])
+
+        children_docs = doc_xml.findall('doc')
+        self.assertEqual(len(children_docs), len(sub_docs))
+
+        self.assertEqual(children_docs[0].find("*[@name='id']").text, sub_docs[0]['id'])
+        self.assertEqual(children_docs[1].find("*[@name='id']").text, sub_docs[1]['id'])
+
     def test_add(self):
         self.assertEqual(len(self.solr.search('doc')), 3)
         self.assertEqual(len(self.solr.search('example')), 2)
@@ -557,7 +614,7 @@ class SolrTestCase(unittest.TestCase):
                 'id': 'doc_7',
                 'title': 'Another example doc',
             },
-        ])
+        ], commit=True)
         # add should default to 'update' handler
         args, kwargs = self.solr._send_request.call_args
         self.assertTrue(args[1].startswith('update/?'))
@@ -567,7 +624,7 @@ class SolrTestCase(unittest.TestCase):
 
         # add should support custom handlers
         with self.assertRaises(SolrError):
-            self.solr.add([], handler='fakehandler')
+            self.solr.add([], handler='fakehandler', commit=True)
         args, kwargs = self.solr._send_request.call_args
         self.assertTrue(args[1].startswith('fakehandler'))
 
@@ -578,19 +635,19 @@ class SolrTestCase(unittest.TestCase):
                       boost={'title': 10.0})
 
         self.solr.add([{'id': 'doc_7', 'title': 'Spam doc doc'}],
-                      boost={'title': 0})
+                      boost={'title': 0}, commit=True)
 
         res = self.solr.search('doc')
         self.assertEqual(len(res), 5)
         self.assertEqual('doc_6', res.docs[0]['id'])
 
-    def test_field_update(self):
+    def test_field_update_inc(self):
         originalDocs = self.solr.search('doc')
         self.assertEqual(len(originalDocs), 3)
         updateList = []
         for i, doc in enumerate(originalDocs):
             updateList.append({'id': doc['id'], 'popularity': 5})
-        self.solr.add(updateList, fieldUpdates={'popularity': 'inc'})
+        self.solr.add(updateList, fieldUpdates={'popularity': 'inc'}, commit=True)
 
         updatedDocs = self.solr.search('doc')
         self.assertEqual(len(updatedDocs), 3)
@@ -601,6 +658,25 @@ class SolrTestCase(unittest.TestCase):
             self.assertEqual(True, all(updatedDoc[k] == originalDoc[k] for k in updatedDoc.keys()
                                        if k not in ['_version_', 'popularity']))
 
+    def test_field_update_set(self):
+        originalDocs = self.solr.search('doc')
+        updated_popularity = 10
+        self.assertEqual(len(originalDocs), 3)
+        updateList = []
+        for i, doc in enumerate(originalDocs):
+            updateList.append({'id': doc['id'], 'popularity': updated_popularity})
+        self.solr.add(updateList, fieldUpdates={'popularity': 'set'}, commit=True)
+
+        updatedDocs = self.solr.search('doc')
+        self.assertEqual(len(updatedDocs), 3)
+        for i, (originalDoc, updatedDoc) in enumerate(zip(originalDocs, updatedDocs)):
+            self.assertEqual(len(updatedDoc.keys()), len(originalDoc.keys()))
+            self.assertEqual(updatedDoc['popularity'], updated_popularity)
+            # TODO: change this to use assertSetEqual:
+            self.assertEqual(True, all(updatedDoc[k] == originalDoc[k] for k in updatedDoc.keys()
+                                       if k not in ['_version_', 'popularity']))
+
+    def test_field_update_add(self):
         self.solr.add([
             {
                 'id': 'multivalued_1',
@@ -612,14 +688,14 @@ class SolrTestCase(unittest.TestCase):
                 'title': 'Multivalued doc 2',
                 'word_ss': ['charlie', 'delta'],
             },
-        ])
+        ], commit=True)
 
         originalDocs = self.solr.search('multivalued')
         self.assertEqual(len(originalDocs), 2)
         updateList = []
         for i, doc in enumerate(originalDocs):
             updateList.append({'id': doc['id'], 'word_ss': ['epsilon', 'gamma']})
-        self.solr.add(updateList, fieldUpdates={'word_ss': 'add'})
+        self.solr.add(updateList, fieldUpdates={'word_ss': 'add'}, commit=True)
 
         updatedDocs = self.solr.search('multivalued')
         self.assertEqual(len(updatedDocs), 2)
@@ -632,7 +708,7 @@ class SolrTestCase(unittest.TestCase):
 
     def test_delete(self):
         self.assertEqual(len(self.solr.search('doc')), 3)
-        self.solr.delete(id='doc_1')
+        self.solr.delete(id='doc_1', commit=True)
         # delete should default to 'update' handler
         args, kwargs = self.solr._send_request.call_args
         self.assertTrue(args[1].startswith('update/?'))
@@ -642,26 +718,54 @@ class SolrTestCase(unittest.TestCase):
         self.assertEqual(len(self.solr.search('type_s:child')), 3)
         self.assertEqual(len(self.solr.search('type_s:grandchild')), 1)
         self.solr.delete(q='price:[0 TO 15]')
-        self.solr.delete(q='type_s:parent')
+        self.solr.delete(q='type_s:parent', commit=True)
         # one simple doc should remain
         # parent documents were also deleted but children remain as orphans
         self.assertEqual(len(self.solr.search('doc')), 1)
         self.assertEqual(len(self.solr.search('type_s:parent')), 0)
         self.assertEqual(len(self.solr.search('type_s:child')), 3)
-        self.solr.delete(q='type_s:child OR type_s:grandchild')
+        self.solr.delete(q='type_s:child OR type_s:grandchild', commit=True)
 
         self.assertEqual(len(self.solr.search('*:*')), 1)
-        self.solr.delete(q='*:*')
+        self.solr.delete(q='*:*', commit=True)
         self.assertEqual(len(self.solr.search('*:*')), 0)
 
-        # Need at least one.
+        # Test delete() with `id' being a list.
+        # Solr's ability to delete parent/children docs by id is simply assumed
+        # and not what's under test here.
+        def leaf_doc(doc):
+            return 'price' in doc and NESTED_DOC_KEY not in doc
+
+        to_delete_docs = list(filter(leaf_doc, self.docs))
+        to_delete_ids = [doc['id'] for doc in to_delete_docs]
+
+        self.solr.add(to_delete_docs)
+        self.solr.commit()
+
+        leaf_q = 'price:[* TO *]'
+        self.assertEqual(len(self.solr.search(leaf_q)), len(to_delete_docs))
+        # Extract a random doc from the list, to later check it wasn't deleted.
+        graced_doc_id = to_delete_ids.pop(random.randint(0, len(to_delete_ids) - 1))
+        self.solr.delete(id=to_delete_ids, commit=True)
+        # There should be only one left, our graced id
+        self.assertEqual(len(self.solr.search(leaf_q)), 1)
+        self.assertEqual(len(self.solr.search('id:%s' % graced_doc_id)), 1)
+        # Now we can wipe the graced document too. None should be left.
+        self.solr.delete(id=graced_doc_id, commit=True)
+        self.assertEqual(len(self.solr.search(leaf_q)), 0)
+
+        # Can't delete when the list of documents is empty
+        self.assertRaises(ValueError, self.solr.delete, id=[None, None, None])
+        self.assertRaises(ValueError, self.solr.delete, id=[None])
+
+        # Need at least one of either `id' or `q'
         self.assertRaises(ValueError, self.solr.delete)
         # Can't have both.
         self.assertRaises(ValueError, self.solr.delete, id='foo', q='bar')
 
         # delete should support custom handlers
         with self.assertRaises(SolrError):
-            self.solr.delete(id='doc_1', handler='fakehandler')
+            self.solr.delete(id='doc_1', handler='fakehandler', commit=True)
         args, kwargs = self.solr._send_request.call_args
         self.assertTrue(args[1].startswith('fakehandler'))
 
@@ -672,7 +776,7 @@ class SolrTestCase(unittest.TestCase):
                 'id': 'doc_6',
                 'title': 'Newly added doc',
             }
-        ], commit=False)
+        ])
         self.assertEqual(len(self.solr.search('doc')), 3)
         self.solr.commit()
         # commit should default to 'update' handler
@@ -680,6 +784,21 @@ class SolrTestCase(unittest.TestCase):
         self.assertTrue(args[1].startswith('update/?'))
         self.assertEqual(len(self.solr.search('doc')), 4)
 
+    def test_can_handles_default_commit_policy(self):
+        expected_commits = [False, True, False]
+        commit_arg = [False, True, None]
+
+        for expected_commit, arg in zip(expected_commits, commit_arg):
+            self.solr.add([
+                {
+                    'id': 'doc_6',
+                    'title': 'Newly added doc',
+                }
+            ], commit=arg)
+            args, _ = self.solr._send_request.call_args
+            committing_in_url = 'commit' in args[1]
+            self.assertEqual(expected_commit, committing_in_url)
+
     def test_overwrite(self):
         self.assertEqual(len(self.solr.search('id:doc_overwrite_1')), 0)
         self.solr.add([
@@ -691,7 +810,7 @@ class SolrTestCase(unittest.TestCase):
                 'id': 'doc_overwrite_1',
                 'title': 'Kim is more awesome.',
             }
-        ], overwrite=False)
+        ], overwrite=False, commit=True)
         self.assertEqual(len(self.solr.search('id:doc_overwrite_1')), 2)
 
         # commit should support custom handlers
@@ -763,6 +882,47 @@ class SolrTestCase(unittest.TestCase):
         # round-trip:
         self.assertEqual(['Test Title ☃☃'], m['title'])
 
+    def test_extract_special_char_in_filename(self):
+        fake_f = StringIO("""
+            <html>
+                <head>
+                    <meta charset="utf-8">
+                    <meta name="haystack-test" content="test 1234">
+                    <title>Test Title ☃&#x2603;</title>
+                </head>
+                    <body>foobar</body>
+            </html>
+        """)
+        fake_f.name = u"test☃.html"
+        extracted = self.solr.extract(fake_f)
+        # extract should default to 'update/extract' handler
+        args, kwargs = self.solr._send_request.call_args
+        self.assertTrue(args[1].startswith('update/extract'))
+
+        # extract should support custom handlers
+        with self.assertRaises(SolrError):
+            self.solr.extract(fake_f, handler='fakehandler')
+        args, kwargs = self.solr._send_request.call_args
+        self.assertTrue(args[1].startswith('fakehandler'))
+
+        # Verify documented response structure:
+        self.assertIn('contents', extracted)
+        self.assertIn('metadata', extracted)
+
+        self.assertIn('foobar', extracted['contents'])
+
+        m = extracted['metadata']
+
+        self.assertEqual([quote(fake_f.name.encode('utf-8'))], m['stream_name'])
+
+        self.assertIn('haystack-test', m, "HTML metadata should have been extracted!")
+        self.assertEqual(['test 1234'], m['haystack-test'])
+
+        # Note the underhanded use of a double snowman to verify both that Tika
+        # correctly decoded entities and that our UTF-8 characters survived the
+        # round-trip:
+        self.assertEqual(['Test Title ☃☃'], m['title'])
+
     def test_full_url(self):
         self.solr.url = 'http://localhost:8983/solr/core0'
         full_url = self.solr._create_full_url(path='/update')
@@ -803,3 +963,49 @@ class SolrTestCase(unittest.TestCase):
         # reset the values to what they were before the test
         self.solr.use_qt_param = before_test_use_qt_param
         self.solr.search_handler = before_test_search_handler
+
+
+class SolrCommitByDefaultTestCase(unittest.TestCase, SolrTestCaseMixin):
+
+    def setUp(self):
+        super(SolrCommitByDefaultTestCase, self).setUp()
+        self.solr = self.get_solr("core0", always_commit=True)
+        self.docs = [
+            {
+                'id': 'doc_1',
+                'title': 'Newly added doc',
+            },
+            {
+                'id': 'doc_2',
+                'title': 'Another example doc',
+            },
+        ]
+
+    def test_does_not_require_commit(self):
+        # add should not require commit arg
+        self.solr.add(self.docs)
+
+        self.assertEqual(len(self.solr.search('doc')), 2)
+        self.assertEqual(len(self.solr.search('example')), 1)
+
+        # update should not require commit arg
+        self.docs[0]['title'] = "Updated Doc"
+        self.docs[1]['title'] = "Another example updated doc"
+        self.solr.add(self.docs, fieldUpdates={'title': 'set'})
+        self.assertEqual(len(self.solr.search('updated')), 2)
+        self.assertEqual(len(self.solr.search('example')), 1)
+
+        # delete should not require commit arg
+        self.solr.delete(q='*:*')
+        self.assertEqual(len(self.solr.search('*')), 0)
+
+    def test_can_handles_default_commit_policy(self):
+        self.solr._send_request = Mock(wraps=self.solr._send_request)
+        expected_commits = [False, True, True]
+        commit_arg = [False, True, None]
+
+        for expected_commit, arg in zip(expected_commits, commit_arg):
+            self.solr.add(self.docs, commit=arg)
+            args, _ = self.solr._send_request.call_args
+            committing_in_url = 'commit' in args[1]
+            self.assertEqual(expected_commit, committing_in_url)
